@@ -47,7 +47,6 @@
 #include "track-info.h"
 #include "goo-player-cd.h"
 #include "dlg-ripper.h"
-#include "dlg-encoder-options.h"
 
 #define GLADE_EXTRACT_FILE "goobox.glade"
 
@@ -68,6 +67,7 @@ typedef struct {
 	GtkWidget   *e_flac_radiobutton;
 	GtkWidget   *e_mp3_radiobutton;
 	GtkWidget   *e_wave_radiobutton;
+	GtkWidget   *e_save_playlist_checkbutton;
 } DialogData;
 
 
@@ -171,17 +171,14 @@ ok_cb (GtkWidget  *widget,
 		file_format = GOO_FILE_FORMAT_WAVE;
 	pref_set_file_format (file_format);
 
+	eel_gconf_set_boolean (PREF_EXTRACT_SAVE_PLAYLIST, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_save_playlist_checkbutton)));
+
 	/**/
 
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_hide (data->dialog); 
 
-	if (file_format != GOO_FILE_FORMAT_WAVE)
-		dlg_encoder_options (data->dialog,
-				     GTK_WINDOW (data->window), 
-				     file_format);
-	else
-		dlg_extract__start_ripping (data->dialog);		
+	dlg_extract__start_ripping (data->dialog);		
 }
 
 
@@ -311,6 +308,7 @@ dlg_extract (GooWindow *window)
 	data->e_flac_radiobutton = glade_xml_get_widget (data->gui, "e_flac_radiobutton");
 	data->e_mp3_radiobutton = glade_xml_get_widget (data->gui, "e_mp3_radiobutton");
 	data->e_wave_radiobutton = glade_xml_get_widget (data->gui, "e_wave_radiobutton");
+	data->e_save_playlist_checkbutton = glade_xml_get_widget (data->gui, "e_save_playlist_checkbutton");
 
 	btn_ok = glade_xml_get_widget (data->gui, "e_okbutton");
 	btn_cancel = glade_xml_get_widget (data->gui, "e_cancelbutton");
@@ -441,6 +439,8 @@ dlg_extract (GooWindow *window)
 	}
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (filetype_btn), TRUE);
 
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->e_save_playlist_checkbutton), eel_gconf_get_boolean (PREF_EXTRACT_SAVE_PLAYLIST, TRUE));
+
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog), 
