@@ -549,6 +549,40 @@ shell_escape (const char *filename)
 }
 
 
+char *
+escape_uri (const char *uri)
+{
+        const char *start = NULL;
+        const char *uri_no_method;
+        char       *method;
+        char       *epath, *euri;
+
+        if (uri == NULL)
+                return NULL;
+
+        start = strstr (uri, "://");
+        if (start != NULL) {
+                uri_no_method = start + strlen ("://");
+                method = g_strndup (uri, start - uri);
+        } else {
+                uri_no_method = uri;
+                method = NULL;
+        }
+
+	epath = gnome_vfs_escape_host_and_path_string (uri_no_method);
+
+        if (method != NULL) {
+                euri = g_strdup_printf ("%s://%s", method, epath);
+                g_free (epath);
+        } else
+                euri = epath;
+
+        g_free (method);
+
+        return euri;
+}
+
+
 static gchar *
 get_terminal ()
 {
@@ -964,7 +998,7 @@ get_last_field (const char *line,
 char *
 get_temp_work_dir (void)
 {
-	char temp_dir_template[] = "/tmp/fr-XXXXXX";
+	char temp_dir_template[] = "/tmp/goo-XXXXXX";
 	g_assert (mkdtemp (temp_dir_template) != NULL);
 	return g_strdup (temp_dir_template);
 }
@@ -1106,3 +1140,27 @@ file_list__get_prev_field (const char *line,
 
 	return g_strndup (f_start + 1, f_end - f_start);
 }
+
+
+GnomeVFSURI *
+new_uri_from_path (const char *path)
+{
+	char        *escaped;
+	char        *uri_txt;
+	GnomeVFSURI *uri;
+
+	escaped = escape_uri (path);
+	if (escaped[0] == '/')
+		uri_txt = g_strconcat ("file://", escaped, NULL);
+	else
+		uri_txt = g_strdup (escaped);
+
+	uri = gnome_vfs_uri_new (uri_txt);
+
+	g_free (uri_txt);
+	g_free (escaped);
+
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	return uri;
+}
