@@ -23,10 +23,12 @@
 #include <config.h>
 #include <string.h>
 #include <gnome.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include "goo-player-info.h"
 #include "goo-marshal.h"
 #include "glib-utils.h"
 #include "goo-stock.h"
+#include "file-utils.h"
 
 #define SPACING 0
 #define TITLE1_FORMAT "<span size='large' weight='bold'>%s</span>"
@@ -36,10 +38,12 @@
 #define PLAYING_FORMAT "[ <small>%s</small> ]"
 #define SCALE_WIDTH 150
 #define COVER_SIZE 80
-#define MIN_WIDTH 400
+#define MIN_WIDTH 440
+#define MIN_CHARS 33
 #define UPDATE_TIMEOUT 50
 
 struct _GooPlayerInfoPrivateData {
+	GooWindow   *window;
 	GooPlayer   *player;
 	GtkWidget   *title1_label;
 	GtkWidget   *title2_label;
@@ -49,6 +53,7 @@ struct _GooPlayerInfoPrivateData {
 	GtkWidget   *time_scale;
 	GtkWidget   *cover_image;
 	GtkWidget   *cover_button;
+	GtkWidget   *cover_sep;
 	GtkTooltips *tips;
 	char         current_time[64];
 	char         total_time[64];
@@ -67,6 +72,12 @@ enum {
 static GtkHBoxClass *parent_class = NULL;
 static guint goo_player_info_signals[LAST_SIGNAL] = { 0 };
 
+enum { TARGET_URL };
+static GtkTargetEntry target_table[] = {
+	{ "text/uri-list", 0, TARGET_URL }
+};
+static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);
+
 static void goo_player_info_class_init  (GooPlayerInfoClass *class);
 static void goo_player_info_init        (GooPlayerInfo *player);
 static void goo_player_info_finalize    (GObject *object);
@@ -294,6 +305,121 @@ cover_button_clicked_cb (GtkWidget     *button,
 }
 
 
+/* -- drag and drop -- */
+
+
+static char *
+get_path_from_url (char *url)
+{
+	GnomeVFSURI *uri;
+	char        *escaped;
+	char        *path;
+
+	if (url == NULL)
+		return NULL;
+
+	uri = gnome_vfs_uri_new (url);
+	escaped = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+	path = gnome_vfs_unescape_string (escaped, NULL);
+
+	gnome_vfs_uri_unref (uri);
+	g_free (escaped);
+
+	return path;
+}
+
+
+static GList *
+get_file_list_from_dnd_data (gchar *url_list)
+{
+	GList    *list = NULL;
+	int       i;
+	char     *url_start, *url_end;
+	gboolean  dnd_valid = FALSE;
+
+	i = 0;
+	url_start = url_list;
+	while (url_list[i] != '\0')	{
+		char *url;
+
+		while ((url_list[i] != '\0')
+		       && (url_list[i] != '\r')
+		       && (url_list[i] != '\n')) i++;
+
+		url_end = url_list + i;
+		if (strncmp (url_start, "file:", 5) == 0) {
+			url_start += 5;
+			if ((url_start[0] == '/') 
+			    && (url_start[1] == '/')) url_start += 2;
+
+			dnd_valid = TRUE;
+
+			url = g_strndup (url_start, url_end - url_start);
+			list = g_list_prepend (list, get_path_from_url (url));
+			g_free (url);
+
+			while ((url_list[i] != '\0')
+			       && ((url_list[i] == '\r') 
+				   || (url_list[i] == '\n'))) i++;
+			url_start = url_list + i;
+
+		} else {
+			while ((url_list[i] != '\0')
+			       && ((url_list[i] == '\r') 
+				   || (url_list[i] == '\n'))) i++;
+			url_start = url_list + i;
+			if (url_list[i] == '\0' && !dnd_valid) {
+				path_list_free (list);
+				return NULL;
+			}
+		}
+	}
+	
+	if (!dnd_valid) {
+		path_list_free (list);
+		return NULL;
+	}
+
+	return g_list_reverse (list);
+}
+
+
+void  
+cover_button_drag_data_received  (GtkWidget          *widget,
+				  GdkDragContext     *context,
+				  gint                x,
+				  gint                y,
+				  GtkSelectionData   *data,
+				  guint               dnd_info,
+				  guint               time,
+				  gpointer            extra_data)
+{
+	GooPlayerInfo *info = extra_data;
+	GList         *list;
+	char          *cover_filename;
+
+	if (! ((data->length >= 0) && (data->format == 8))) {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+		return;
+	}
+
+	gtk_drag_finish (context, TRUE, FALSE, time);
+
+	list = get_file_list_from_dnd_data ((char *)data->data);
+	if (list == NULL)
+		return;
+
+	cover_filename = list->data;
+	if (cover_filename == NULL)
+		return;
+
+	goo_window_set_cover_image (info->priv->window, cover_filename);
+
+	if (list != NULL) 
+		path_list_free (list);
+}
+
+
 static void 
 goo_player_info_init (GooPlayerInfo *info)
 {
@@ -328,6 +454,20 @@ goo_player_info_init (GooPlayerInfo *info)
 	gtk_misc_set_alignment (GTK_MISC (priv->title3_label), 0.0, 0.5);
 	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), TRUE);
 
+#ifdef HAVE_GTK_2_5
+	gtk_label_set_ellipsize (GTK_LABEL (priv->title1_label),
+				 PANGO_ELLIPSIZE_END);
+	gtk_label_set_width_chars (GTK_LABEL (priv->title2_label), MIN_CHARS);
+
+	gtk_label_set_ellipsize (GTK_LABEL (priv->title2_label),
+				 PANGO_ELLIPSIZE_END);
+	gtk_label_set_width_chars (GTK_LABEL (priv->title2_label), MIN_CHARS);
+	
+	gtk_label_set_ellipsize (GTK_LABEL (priv->title3_label),
+				 PANGO_ELLIPSIZE_END);
+	gtk_label_set_width_chars (GTK_LABEL (priv->title3_label), MIN_CHARS);
+#endif
+
 	/* Time */
 
 	time_box = gtk_hbox_new (FALSE, 6);
@@ -363,6 +503,16 @@ goo_player_info_init (GooPlayerInfo *info)
 			  G_CALLBACK (cover_button_clicked_cb),
 			  info);
 
+	gtk_drag_dest_set (priv->cover_button,
+			   GTK_DEST_DEFAULT_ALL,
+			   target_table, n_targets,
+			   GDK_ACTION_COPY);
+	
+	g_signal_connect (G_OBJECT (priv->cover_button), 
+			  "drag_data_received",
+			  G_CALLBACK (cover_button_drag_data_received), 
+			  info);
+
 	priv->cover_image = gtk_image_new_from_stock (GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
 	gtk_widget_set_size_request (priv->cover_image, COVER_SIZE, COVER_SIZE);
 	gtk_widget_show (priv->cover_image);
@@ -370,11 +520,10 @@ goo_player_info_init (GooPlayerInfo *info)
 
 	gtk_box_pack_start (GTK_BOX (info), priv->cover_button, FALSE, FALSE, 0);
 
-	/*FIXME
-	gtk_box_pack_start (GTK_BOX (info), 
-			    gtk_vseparator_new (),
-			    FALSE, FALSE, 0);
-	*/
+	priv->cover_sep = gtk_vseparator_new ();
+	gtk_box_pack_start (GTK_BOX (info), priv->cover_sep, FALSE, FALSE, 6);
+	gtk_widget_show (priv->cover_sep);
+	gtk_widget_set_no_show_all (priv->cover_sep, TRUE);
 
 	/**/
 
@@ -435,11 +584,16 @@ goo_player_info_finalize (GObject *object)
 
 
 GtkWidget *
-goo_player_info_new (GooPlayer *player)
+goo_player_info_new (GooWindow *window,
+		     GooPlayer *player)
 {
 	GooPlayerInfo *info;
 	
 	info = GOO_PLAYER_INFO (g_object_new (GOO_TYPE_PLAYER_INFO, NULL));
+
+	if (window != NULL) 
+		info->priv->window = window;
+
 	if (player != NULL) {
 		g_object_ref (player);
 		info->priv->player = player;
@@ -622,8 +776,11 @@ void
 goo_player_info_set_cover (GooPlayerInfo  *info,
 			   GdkPixbuf      *cover)
 {
-	if (cover != NULL)
+	if (cover != NULL) {
 		gtk_image_set_from_pixbuf (GTK_IMAGE (info->priv->cover_image), cover);
-	else
+		gtk_widget_hide (info->priv->cover_sep);
+	} else {
 		gtk_image_set_from_stock (GTK_IMAGE (info->priv->cover_image), GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
+		gtk_widget_show (info->priv->cover_sep);
+	}
 }
